@@ -39,6 +39,15 @@ def label_internal_nodes_ete(ete_tree: Tree) -> None:
 
 
 def edge_from_node_name(ete_tree: Tree, node_name) -> int:
+    """
+    Returns the number corresponding to a node's proximal edge (i.e. the edge connecting the node to its parent)
+
+    Note: this algorithm is only suitable for complete trees, not subtrees!
+
+    :param ete_tree: An ETE3 Tree instance where all nodes have names that can be matches
+    :param node_name: The name of the node to retrieve the edge of
+    :return: An integer representing the name of the node's edge
+    """
     edge_name = 0
     if len(ete_tree.children) > 2:
         ete_tree.resolve_polytomy(recursive=False)
@@ -148,6 +157,7 @@ def map_internal_nodes_leaves(tree: str) -> dict:
     node_stack = list()
     leaf_stack = list()
     x = 0
+    current_node = 0
     num_buffer = ""
     while x < len(no_length_tree):
         c = no_length_tree[x]
@@ -189,7 +199,17 @@ def map_internal_nodes_leaves(tree: str) -> dict:
         logging.error("Node stack not empty by end of loading internal-node map:\n" + str(node_stack) + "\n")
         sys.exit(11)
 
-    # validate_internal_node_map(node_map)
+    # Ensure all the leaves were popped in case the tree was unrooted
+    while leaf_stack:
+        current_node += 1
+        try:
+            node_map[current_node] = node_map[leaf_stack.pop()] + node_map[leaf_stack.pop()]
+        except IndexError:
+            logging.error("Tried to generate leaf-to-internal node map from multifurcating tree.\n")
+            sys.exit(11)
+        if leaf_stack:
+            leaf_stack.append(current_node)
+
     return node_map
 
 
