@@ -198,7 +198,13 @@ class PQuery:
     def name_placed_sequence(self) -> None:
         names = set()
         for pplace in self.placements:  # type: PhyloPlace
-            names.add(pplace.name)
+            # Handles different JPlace formats
+            try:
+                names.add(pplace.name)
+            except TypeError:
+                if isinstance(pplace.name, list):
+                    for n in pplace.name:
+                        names.add(n)
 
         if len(names) > 1:
             logging.error("Multiple names encountered for a single PQuery:\n{}\n".format(','.join(names)))
@@ -320,7 +326,7 @@ class PQuery:
 
         return confident_assignment
 
-    def calculate_consensus_placement(self, labelled_tree: Tree, min_aelw=0.5) -> None:
+    def calculate_consensus_placement(self, labelled_tree: Tree, min_aelw=0.66) -> None:
         """
         Often times, a single PQuery (query sequence mapped onto a phylogeny) may be inserted into the phylogeny
         at multiple edges with similar likelihood.
@@ -329,6 +335,9 @@ class PQuery:
         ancestor of all descendent placements that contributed to the new taxonomic assignment. In the latter case, the
         distal, pendant and mean-tip lengths are all set to 0.0.
 
+        All placements that are found to have contributed to the final consensus placement are removed and replaced
+        with the single consensus placement.
+
         :return: None
         """
         # If the number of placements is one, set the consensus placement to the only placement
@@ -340,13 +349,19 @@ class PQuery:
             return
 
         dist_ratio = 0.49
-        aelw_sum = 0.0
         node_aelw_map = {}  # Maps internal node names to accumulated ELW values
         taxon_aelw_map = {}  # Maps taxon names to accumulated ELW values
         node_name_map = {}  # Maps internal node names to TreeNode instances
         taxon_name_map = {}  # Maps taxon names to Taxon instances
         contributors = []
-        for pplace in self.placements:  # type: PhyloPlace
+        # Sort to pop the placements in order of biggest likelihood weight ratio to smallest
+        self.placements = sorted(self.placements, key=lambda x: float(x.like_weight_ratio))
+        self.consensus_placement = PhyloPlace()
+        self.consensus_placement.name = self.placements[-1].name
+        self.consensus_placement.likelihood = self.placements[-1].likelihood
+        while self.placements:
+            # Remove the placements contributing to the aELW and replace with the consensus placement
+            pplace = self.placements.pop(-1)  # type: PhyloPlace
             up_node, down_node = get_ete_edge(labelled_tree, pplace.edge_num)
             parent, child = up_node.taxon, down_node.taxon
             # Ensure each of the taxon in a lineage is in the aELW dictionary
@@ -356,26 +371,30 @@ class PQuery:
                     node_aelw_map[node.name] = 0
 
             if parent == child:
-                node_aelw_map[up_node.name] += pplace.like_weight_ratio
-                node_aelw_map[down_node.name] += pplace.like_weight_ratio
+                node_aelw_map[up_node.name] += (pplace.like_weight_ratio/2)
+                node_aelw_map[down_node.name] += (pplace.like_weight_ratio/2)
             else:
                 node_aelw_map[up_node.name] += (pplace.like_weight_ratio * dist_ratio)
                 node_aelw_map[down_node.name] += (pplace.like_weight_ratio * (1-dist_ratio))
 
-        for node_name in reversed(sorted(node_aelw_map, key=lambda x: node_aelw_map[x])):
-            aelw_sum += node_aelw_map[node_name]
-            contributors.append(node_name_map[node_name])
-            if aelw_sum >= min_aelw:
+            # Check whether the minimum likelihood weight has been reached
+            self.consensus_placement.like_weight_ratio = 0.0
+            contributors.clear()
+            for node_name in reversed(sorted(node_aelw_map, key=lambda x: node_aelw_map[x])):
+                self.consensus_placement.like_weight_ratio += node_aelw_map[node_name]
+                contributors.append(node_name_map[node_name])
+                if self.consensus_placement.like_weight_ratio >= min_aelw:
+                    break
+            if self.consensus_placement.like_weight_ratio >= min_aelw:
                 break
 
         # Find the LCA edge of all placements that contributed to the taxonomic assignment
         node_lca = contributors[0].get_common_ancestor(contributors)  # type: Tree
         # Populate the new placement's attributes
-        c_pplace = PhyloPlace()
-        c_pplace.distal_length, c_pplace.pendant_length, c_pplace.mean_tip_length = 0.0, 0.0, 0.0
-        c_pplace.edge_num = edge_from_node_name(labelled_tree, node_name=node_lca.name)
-        c_pplace.like_weight_ratio = aelw_sum
-        self.consensus_placement = c_pplace
+        self.consensus_placement.distal_length, self.consensus_placement.pendant_length, self.consensus_placement.mean_tip_length = 0.0, 0.0, 0.0
+        self.consensus_placement.edge_num = edge_from_node_name(labelled_tree, node_name=node_lca.name)
+        # Replace the placements that were used in the LCA with the consensus placement
+        self.placements.append(self.consensus_placement)
 
         # Sum the likelihood weights across the different taxa
         for node in contributors:
@@ -431,13 +450,12 @@ class PQuery:
         return
 
 
-def assignments_to_treesaps(classified_lines: list, refpkg_dict: dict) -> dict:
+def assignments_to_treesaps(classified_lines: list) -> dict:
     """
     Used for converting the TreeSAPP-assignment information of classified sequences (found in self.classifications)
     into JPlace instances such that these can be reproducibly modified and written again, if needed.
 
     :param classified_lines: A list of lists. Each sub-list represents a line from self.classifications
-    :param refpkg_dict: A dictionary of MarkerBuild instances indexed by their RefPkg codes
     :return: A dictionary of JPlace instances, indexed by their respective names (ReferencePackage.prefix)
     """
     pqueries = dict()
@@ -446,12 +464,13 @@ def assignments_to_treesaps(classified_lines: list, refpkg_dict: dict) -> dict:
         pquery = PQuery()
         con_place = PhyloPlace()
         try:
-            _, pquery.place_name, pquery.ref_name, pquery.start, pquery.end, pquery.recommended_lineage, \
+            _, pquery.seq_name, pquery.ref_name, pquery.start, pquery.end, pquery.recommended_lineage, \
             _, con_place.edge_num, pquery.evalue, con_place.like_weight_ratio, pquery.avg_evo_dist, pquery.distances = fields
         except ValueError:
             logging.error("Bad line in classification table:\n" +
                           '\t'.join(fields) + "\n")
             sys.exit(21)
+        pquery.place_name = "{}|{}|{}_{}".format(pquery.seq_name, pquery.ref_name, pquery.start, pquery.end)
         pquery.end = int(pquery.end)
         pquery.start = int(pquery.start)
         pquery.seq_len = pquery.end - pquery.start
@@ -459,11 +478,10 @@ def assignments_to_treesaps(classified_lines: list, refpkg_dict: dict) -> dict:
         con_place.distal_length, con_place.pendant_length, con_place.mean_tip_length = [float(d) for d in
                                                                                         pquery.distances.split(',')]
         pquery.consensus_placement = con_place
-        refpkg = refpkg_dict[pquery.ref_name]  # type: refpkg.ReferencePackage
         try:
-            pqueries[refpkg.prefix].append(pquery)
+            pqueries[pquery.ref_name].append(pquery)
         except KeyError:
-            pqueries[refpkg.prefix] = [pquery]
+            pqueries[pquery.ref_name] = [pquery]
     return pqueries
 
 
